from abc import ABCMeta, abstractmethod
from typing import Dict, Union

from nidhoggr.core.response import StatusResponse, TextureStatusResponse, ErrorResponse
from nidhoggr.core.texture import TextureUploadRequest, TextureRequest, TextureResponse
from nidhoggr.core.user import User


class BaseTextureRepo(metaclass=ABCMeta):
    variant: str

    @abstractmethod
    def get(self, *, request: TextureRequest) -> Union[ErrorResponse, TextureResponse]:
        pass

    @abstractmethod
    def upload(self, *, request: TextureUploadRequest) -> Union[ErrorResponse, TextureStatusResponse]:
        pass

    @abstractmethod
    def clear(self, *, request: TextureRequest) -> Union[ErrorResponse, TextureStatusResponse]:
        pass


class BaseUserRepo(metaclass=ABCMeta):
    EMPTY_USER = User(
        uuid="8de67985-09c5-443b-be60-8fa552560e0e",
        login="empty",
        email="empty@example.com",
        synthetic=True
    )

    @abstractmethod
    def get_user(self, **kw: Dict[str, str]) -> Union[ErrorResponse, User]:
        pass

    @abstractmethod
    def check_password(self, *, clean: str, uuid: str) -> Union[ErrorResponse, StatusResponse]:
        pass

    @abstractmethod
    def save_user(self, *, user: User) -> Union[ErrorResponse, User]:
        pass



from enum import Enum
from typing import Optional, Dict, List, Tuple

from pydantic import UUID4
from pydantic.main import BaseModel

TextureMeta = Optional[Dict[str, str]]


class TextureType(Enum):
    SKIN = "SKIN"
    CAPE = "CAPE"
    ELYTRA = "ELYTRA"


class TextureItem(BaseModel):
    url: str
    metadata: TextureMeta = None

    class Config:
        allow_mutation = False


class TextureRequest(BaseModel):
    uuid: UUID4
    texture_types: List[TextureType] = [TextureType.SKIN]

    class Config:
        allow_mutation = False


class TextureUploadRequest(BaseModel):
    uuid: UUID4
    data: str
    kind: TextureType
    metadata: TextureMeta

    class Config:
        allow_mutation = False


class TextureResponse(BaseModel):
    textures: List[Tuple[TextureType, TextureItem]]

    class Config:
        allow_mutation = False
